function [K,E]=rsqlocus(a,b,q,r,nn,e)
%RSQLOCUS  Root square locus for continuous-time systems.
%       RSQLOCUS(A,B,Q,R) computes and plots the locus of the eigenvalues
%       of the hamiltonian matrix associated with the Linear Quadratic
%       problem:
%       Find a stabilizing state feedback Kc such that u=-Kc*x minimizes:
%
%               J = Integral {x'Qx + ku'Ru} dt
%
%       with:
%               .
%               x = Ax + Bu 
%
%       for a set of gains k automatically selected.
%       The particular nominal case (k=1) is marked with blue '+'.
%          
%       RSQLOCUS(A,B,Q,R,N) takes into account state-input cross coupling
%       weights in the quadratic index
%
%               J = Integral {x'Qx + ku'Ru + 2*x'Nu}
%          
%       [GAIN,E]=RSQLOCUS(A,B,Q,R,N,K) allows the vector K of gains k 
%       to be taken into account and returns:
%           - a matrix GAIN  (LENGTH(K) rows and LENGTH(A) culums)
%             with the values of state feedback gains (u=-GAIN.x)
%             for the corresponding values of K
%           - a matrix E (LENGTH(K) columns and LENGTH(A) rows)
%             with closed-loop (and stable) eigenvalues for the 
%             corresponding values of K.
%          
%       The Hamiltonian matrix (for k=1) reads:
%                   |    -1           -1  |
%                   |A-BR  N'      -BR  B'|
%               H = |                     |
%                   |    -1        -1     |
%                   |-Q+NR  N'   NR  B'-A'|
%          
%       See also: LQR, RLOCUSP


% D. Alazard 01/94
% Copyright (c) 1993-2000 ONERA/DCSD, All Rights Reserved.

error(nargchk(4,6,nargin));
error(abcdchk(a,b));
if ~length(a) | ~length(b)
        error('A and B matrices cannot be empty.')
end

[m,n] = size(a);
[mb,nb] = size(b);
[mq,nq] = size(q);
if (m ~= mq) | (n ~= nq) 
	error('A and Q must be the same size');
end
[mr,nr] = size(r);
if (mr ~= nr) | (nb ~= mr)
	error('B and R must be consistent');
end

if (nargin == 5) | (nargin == 6)
	[mn,nnn] = size(nn);
	if (mn ~= m) | (nnn ~= nr)
		error('N must be consistent with Q and R');
	end
else
	nn = zeros(m,nb);
end


aa=[a 0*a;-q -a'];
rm1=inv(r);
bb=[b;-nn]*sqrtm(rm1);
cc=sqrtm(rm1)*[nn' b'];
dd=0*ones(size(cc*bb));
if (nargin==4) | (nargin==5)
	rlocusp(aa,bb,cc,dd)
end
if (nargin==6)
	rlocusp(aa,bb,cc,dd,1./e);
	for ii=1:length(e)
		kk=1+nb*(ii-1);
		[v,d]=eig(aa-1./e(ii)*bb*cc);
		d=diag(d);
		[F,index]=sort(real(d));
                E(:,ii)=d(index(1:n));
		chi=v(1:n,index(1:n));
		lambda = v((n+1):(2*n),index(1:n));
		s = -real(lambda/chi);
		K(kk:kk+nb-1,:) = (e(ii)*r)\(nn'+b'*s);
	end;
end;
